import numpy as np


red = np.asarray([1, 0, 0], dtype='f')
green = np.asarray([0, 1, 0], dtype='f')
blue = np.asarray([0, 0, 1], dtype='f')

yellow = np.asarray([1, 1, 0], dtype='f')

orange = np.asarray([1, 0.5, 0], dtype='f')

white = np.asarray([1, 1, 1], dtype='f')
black = np.asarray([0, 0, 0], dtype='f')

dark_red = np.asarray([0.5, 0, 0], dtype='f')

light_yellow = np.asarray([1, 1, 0.5], dtype='f')
# light_blue = np.asarray([0, 0.5, 1], dtype='f')
